@@ -61,7 +61,8 @@
 #define DESKTOP_ICONS_SHOW_TRASH             "/desktop-icons/file-icons/show-trash"
 #define DESKTOP_ICONS_SHOW_FILESYSTEM        "/desktop-icons/file-icons/show-filesystem"
 #define DESKTOP_ICONS_SHOW_REMOVABLE         "/desktop-icons/file-icons/show-removable"
-#define DESKTOP_ICONS_MAX_TEMPLATE_FILES     "/desktop-icons/file-icons/max-template-files"
+
+#define DESKTOP_MENU_MAX_TEMPLATE_FILES     "/desktop-menu/max-template-files"
 
 /**
  * File information namespaces queried for #GFileInfo objects.
@@ -90,9 +91,6 @@ gboolean xfdesktop_image_file_is_valid(const gchar *filename);
 
 gchar *xfdesktop_get_file_mimetype(const gchar *file);
 
-gboolean xfdesktop_check_is_running(Window *xid);
-void xfdesktop_send_client_message(Window xid, const gchar *msg);
-
 gint xfce_translate_image_styles(gint input);
 
 guint32 xfdesktop_popup_keyboard_grab_available(GdkWindow *win);
