@@ -70,6 +70,7 @@
 #define WINLIST_SHOW_STICKY_WIN_ONCE_PROP    "/windowlist-menu/show-sticky-once"
 #define WINLIST_SHOW_WS_NAMES_PROP           "/windowlist-menu/show-workspace-names"
 #define WINLIST_SHOW_WS_SUBMENUS_PROP        "/windowlist-menu/show-submenus"
+#define WINLIST_SHOW_ADD_REMOVE_WORKSPACES_PROP "/windowlist-menu/show-add-remove-workspaces"
 
 #define DESKTOP_ICONS_STYLE_PROP             "/desktop-icons/style"
 #define DESKTOP_ICONS_ICON_SIZE_PROP         "/desktop-icons/icon-size"
@@ -99,7 +100,9 @@ typedef struct
      * wnck_screen_get_active_workspace sometimes has to return NULL. */
     gint active_workspace;
 
-    GtkWidget *frame_image_list;
+    GtkWidget *infobar;
+    GtkWidget *infobar_label;
+    GtkWidget *label_header;
     GtkWidget *image_iconview;
     GtkWidget *btn_folder;
     GtkWidget *chk_apply_to_all;
@@ -680,6 +683,14 @@ xfdesktop_settings_update_iconview_frame_name(AppearancePanel *panel,
             } else {
                 g_snprintf(buf, sizeof(buf), _("Wallpaper for Monitor %d"), panel->monitor);
             }
+
+            /* This is for the infobar letting the user know how to configure
+             * multiple monitor setups */
+            gtk_label_set_text(GTK_LABEL(panel->infobar_label),
+                               _("You are using more than one display, "
+                                 "move this dialog to the display you "
+                                 "want to edit the settings for."));
+            gtk_widget_set_visible(panel->infobar, TRUE);
         } else {
             /* Multi-monitor per workspace wallpaper */
             if(panel->monitor_name) {
@@ -691,18 +702,37 @@ xfdesktop_settings_update_iconview_frame_name(AppearancePanel *panel,
                            _("Wallpaper for %s on Monitor %d"),
                            workspace_name, panel->monitor);
             }
+
+            /* This is for the infobar letting the user know how to configure
+             * multiple monitor/workspace setups */
+            gtk_label_set_text(GTK_LABEL(panel->infobar_label),
+                               _("You are using more than one display, "
+                                 "move this dialog to the display and "
+                                 "workspace you want to edit the settings for."));
+            gtk_widget_set_visible(panel->infobar, TRUE);
         }
     } else {
         if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(panel->chk_apply_to_all))) {
             /* Single monitor and single workspace */
             g_snprintf(buf, sizeof(buf), _("Wallpaper for my desktop"));
+
+            /* No need for the infobar */
+            gtk_widget_set_visible(panel->infobar, FALSE);
         } else {
             /* Single monitor and per workspace wallpaper */
             g_snprintf(buf, sizeof(buf), _("Wallpaper for %s"), workspace_name);
+
+            /* This is for the infobar letting the user know how to configure
+             * multiple workspace setups */
+            gtk_label_set_text(GTK_LABEL(panel->infobar_label),
+                               _("Move this dialog to the workspace you "
+                                 "want to edit the settings for."));
+            gtk_widget_set_visible(panel->infobar, TRUE);
         }
     }
 
-    gtk_frame_set_label(GTK_FRAME(panel->frame_image_list), buf);
+    /* This label is for which workspace/monitor we're on */
+    gtk_label_set_text(GTK_LABEL(panel->label_header), buf);
 
     g_free(workspace_name);
 }
@@ -1619,7 +1649,7 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
     GtkWidget *appearance_container, *chk_custom_font_size,
               *spin_font_size, *w, *box, *spin_icon_size,
               *chk_show_thumbnails, *chk_single_click, *appearance_settings,
-              *chk_show_tooltips, *spin_tooltip_size, *bnt_exit;
+              *chk_show_tooltips, *spin_tooltip_size, *bnt_exit, *content_area;
     GtkBuilder *appearance_gxml;
     AppearancePanel *panel = g_new0(AppearancePanel, 1);
     GError *error = NULL;
@@ -1721,15 +1751,29 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
     appearance_settings = GTK_WIDGET(gtk_builder_get_object(appearance_gxml,
                                                             "alignment_settings"));
 
-    panel->frame_image_list = GTK_WIDGET(gtk_builder_get_object(appearance_gxml,
-                                                                "frame_image_list"));
-
-    gtk_table_attach_defaults(GTK_TABLE(appearance_container),
-                             appearance_settings, 0,1,0,1);
+    /* Add the background tab widgets to the main window and don't display the
+     * notebook label/tab */
+    gtk_notebook_append_page(GTK_NOTEBOOK(appearance_container),
+                             appearance_settings, NULL);
+    gtk_notebook_set_show_tabs(GTK_NOTEBOOK(appearance_container), FALSE);
 
     /* icon view area */
-    panel->frame_image_list = GTK_WIDGET(gtk_builder_get_object(appearance_gxml,
-                                                                "frame_image_list"));
+    panel->infobar = GTK_WIDGET(gtk_builder_get_object(appearance_gxml,
+                                                       "infobar_header"));
+
+    panel->infobar_label = gtk_label_new("This is some text");
+    gtk_widget_set_no_show_all(panel->infobar, TRUE);
+    gtk_widget_show(panel->infobar_label);
+    gtk_widget_show(panel->infobar);
+    gtk_widget_set_visible(panel->infobar, FALSE);
+
+    /* Add the panel's infobar label to the infobar, with this setup
+     * it's easy to update the text for the infobar. */
+    content_area = gtk_info_bar_get_content_area(GTK_INFO_BAR(panel->infobar));
+    gtk_container_add(GTK_CONTAINER(content_area), panel->infobar_label);
+
+    panel->label_header = GTK_WIDGET(gtk_builder_get_object(appearance_gxml,
+                                                            "label_header"));
 
     panel->image_iconview = GTK_WIDGET(gtk_builder_get_object(appearance_gxml,
                                                               "iconview_imagelist"));
@@ -1837,6 +1881,11 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
                            gtk_builder_get_object(main_gxml, "chk_show_winlist_sticky_once"),
                            "active");
 
+    xfconf_g_property_bind(channel, WINLIST_SHOW_ADD_REMOVE_WORKSPACES_PROP,
+                           G_TYPE_BOOLEAN,
+                           gtk_builder_get_object(main_gxml, "chk_show_app_remove_workspaces"),
+                           "active");
+
     w = GTK_WIDGET(gtk_builder_get_object(main_gxml, "chk_show_winlist_ws_names"));
     xfconf_g_property_bind(channel, WINLIST_SHOW_WS_NAMES_PROP, G_TYPE_BOOLEAN,
                            G_OBJECT(w), "active");
