@@ -40,6 +40,7 @@ typedef struct _XfceBackdropPriv XfceBackdropPriv;
 
 typedef enum
 {
+    XFCE_BACKDROP_IMAGE_INVALID = -1,
     XFCE_BACKDROP_IMAGE_NONE = 0,
     XFCE_BACKDROP_IMAGE_CENTERED,
     XFCE_BACKDROP_IMAGE_TILED,
@@ -51,12 +52,25 @@ typedef enum
 
 typedef enum
 {
+    XFCE_BACKDROP_COLOR_INVALID = -1,
     XFCE_BACKDROP_COLOR_SOLID = 0,
     XFCE_BACKDROP_COLOR_HORIZ_GRADIENT,
     XFCE_BACKDROP_COLOR_VERT_GRADIENT,
     XFCE_BACKDROP_COLOR_TRANSPARENT,
 } XfceBackdropColorStyle;
 
+typedef enum
+{
+    XFCE_BACKDROP_PERIOD_INVALID = -1,
+    XFCE_BACKDROP_PERIOD_SECONDS = 0,
+    XFCE_BACKDROP_PERIOD_MINUES,
+    XFCE_BACKDROP_PERIOD_HOURS,
+    XFCE_BACKDROP_PERIOD_STARTUP,
+    XFCE_BACKDROP_PERIOD_HOURLY,
+    XFCE_BACKDROP_PERIOD_DAILY,
+    XFCE_BACKDROP_PERIOD_CHRONOLOGICAL,
+} XfceBackdropCyclePeriod;
+
 struct _XfceBackdrop
 {
     GObject gobject;
@@ -116,6 +130,11 @@ void xfce_backdrop_set_cycle_backdrop    (XfceBackdrop *backdrop,
                                           gboolean cycle_backdrop);
 gboolean xfce_backdrop_get_cycle_backdrop(XfceBackdrop *backdrop);
 
+void xfce_backdrop_set_cycle_period      (XfceBackdrop *backdrop,
+                                          XfceBackdropCyclePeriod period);
+XfceBackdropCyclePeriod xfce_backdrop_get_cycle_period
+                                         (XfceBackdrop *backdrop);
+
 void xfce_backdrop_set_cycle_timer       (XfceBackdrop *backdrop,
                                           guint cycle_timer);
 guint xfce_backdrop_get_cycle_timer      (XfceBackdrop *backdrop);
@@ -124,6 +143,10 @@ void xfce_backdrop_set_random_order      (XfceBackdrop *backdrop,
                                           gboolean random_order);
 gboolean xfce_backdrop_get_random_order  (XfceBackdrop *backdrop);
 
+void xfce_backdrop_set_cache_pixbuf      (XfceBackdrop *backdrop,
+                                          gboolean cache_pixbuf);
+gboolean xfce_backdrop_get_cache_pixbuf  (XfceBackdrop *backdrop);
+
 GdkPixbuf *xfce_backdrop_get_pixbuf      (XfceBackdrop *backdrop);
 
 void xfce_backdrop_generate_async        (XfceBackdrop *backdrop);
