@@ -304,11 +304,6 @@ set_real_root_window_pixmap(GdkScreen *gscreen,
             gdk_atom_intern("_XROOTPMAP_ID", FALSE),
             gdk_atom_intern("PIXMAP", FALSE), 32,
             GDK_PROP_MODE_REPLACE, (guchar *)&xid, 1);
-    /* set this other property because someone might need it sometime. */
-    gdk_property_change(groot,
-            gdk_atom_intern("ESETROOT_PMAP_ID", FALSE),
-            gdk_atom_intern("PIXMAP", FALSE), 32,
-            GDK_PROP_MODE_REPLACE, (guchar *)&xid, 1);
     /* and set the root window's BG pixmap, because aterm is somewhat lame. */
     gdk_window_set_back_pixmap(groot, pmap, FALSE);
     /* there really should be a standard for this crap... */
@@ -349,8 +344,6 @@ create_bg_pixmap(GdkScreen *gscreen, gpointer user_data)
     if(!GDK_IS_PIXMAP(desktop->priv->bg_pixmap))
         return NULL;
 
-    set_real_root_window_pixmap(desktop->priv->gscreen,
-                                desktop->priv->bg_pixmap);
     gdk_window_set_back_pixmap(gtk_widget_get_window(GTK_WIDGET(desktop)),
                                desktop->priv->bg_pixmap, FALSE);
 
@@ -517,6 +510,7 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
 
         g_object_unref(G_OBJECT(pix));
         cairo_destroy(cr);
+        gtk_widget_show(GTK_WIDGET(desktop));
     }
 
     if(clip_region != NULL)
@@ -1128,15 +1122,17 @@ xfce_desktop_unrealize(GtkWidget *widget)
     gdk_flush();
     gdk_error_trap_pop();
 
-    g_object_unref(G_OBJECT(desktop->priv->bg_pixmap));
-    desktop->priv->bg_pixmap = NULL;
+    if(desktop->priv->bg_pixmap) {
+        g_object_unref(G_OBJECT(desktop->priv->bg_pixmap));
+        desktop->priv->bg_pixmap = NULL;
+    }
     
     gtk_window_set_icon(GTK_WINDOW(widget), NULL);
-    
+
     gtk_style_detach(gtk_widget_get_style(widget));
     g_object_unref(G_OBJECT(gtk_widget_get_window(widget)));
     gtk_widget_set_window(widget, NULL);
-    
+
     gtk_selection_remove_all(widget);
     
     gtk_widget_set_realized(widget, FALSE);
