@@ -84,6 +84,7 @@ struct _XfceWorkspacePriv
     guint nbackdrops;
     gboolean xinerama_stretch;
     XfceBackdrop **backdrops;
+    gboolean cache_pixbufs;
 
     gulong *first_color_id;
     gulong *second_color_id;
@@ -118,6 +119,14 @@ static void xfce_workspace_remove_backdrops(XfceWorkspace *workspace);
 
 G_DEFINE_TYPE(XfceWorkspace, xfce_workspace, G_TYPE_OBJECT)
 
+
+/**
+ * xfce_workspace_get_xinerama_stretch:
+ * @workspace: An #XfceWorkspace.
+ *
+ * returns whether the first backdrop is set to spanning screens since it's
+ * the only backdrop where that setting is applicable.
+ **/
 gboolean
 xfce_workspace_get_xinerama_stretch(XfceWorkspace *workspace)
 {
@@ -128,36 +137,62 @@ xfce_workspace_get_xinerama_stretch(XfceWorkspace *workspace)
     return xfce_backdrop_get_image_style(workspace->priv->backdrops[0]) == XFCE_BACKDROP_IMAGE_SPANNING_SCREENS;
 }
 
+static void
+xfce_workspace_change_backdrop(XfceWorkspace *workspace,
+                               XfceBackdrop *backdrop,
+                               const gchar *backdrop_file)
+{
+    XfconfChannel *channel = workspace->priv->channel;
+    char buf[1024];
+    gchar *monitor_name = NULL;
+    guint i, monitor_num;
+
+    g_return_if_fail(workspace->priv->nbackdrops > 0);
+
+    TRACE("entering");
+
+    /* Find out which monitor we're on */
+    for(i = 0; i < workspace->priv->nbackdrops; ++i) {
+        if(backdrop == workspace->priv->backdrops[i]) {
+            monitor_num = i;
+            break;
+        }
+    }
+
+    monitor_name = gdk_screen_get_monitor_plug_name(workspace->priv->gscreen,
+                                                    monitor_num);
+
+    /* Get the backdrop's image property */
+    if(monitor_name == NULL) {
+        g_snprintf(buf, sizeof(buf), "%smonitor%d/workspace%d/last-image",
+                   workspace->priv->property_prefix, monitor_num, workspace->priv->workspace_num);
+    } else {
+        g_snprintf(buf, sizeof(buf), "%smonitor%s/workspace%d/last-image",
+                   workspace->priv->property_prefix, monitor_name, workspace->priv->workspace_num);
+
+        g_free(monitor_name);
+    }
+
+    /* Update the property so that xfdesktop won't show the same image every
+     * time it starts up when the user wants it to cycle different images */
+    xfconf_channel_set_string(channel, buf, backdrop_file);
+}
+
 static void
 backdrop_cycle_cb(XfceBackdrop *backdrop, gpointer user_data)
 {
-    const gchar* backdrop_file;
-    gchar *new_backdrop = NULL;
     XfceWorkspace *workspace = XFCE_WORKSPACE(user_data);
+    const gchar *new_backdrop;
 
     TRACE("entering");
 
     g_return_if_fail(XFCE_IS_BACKDROP(backdrop));
 
-    backdrop_file = xfce_backdrop_get_image_filename(backdrop);
+    new_backdrop = xfce_backdrop_get_image_filename(backdrop);
 
-    if(backdrop_file == NULL)
-        return;
-
-    if(xfce_backdrop_get_random_order(backdrop)) {
-        DBG("Random! current file: %s", backdrop_file);
-        new_backdrop = xfdesktop_backdrop_choose_random(backdrop_file);
-    } else {
-        DBG("Next! current file: %s", backdrop_file);
-        new_backdrop = xfdesktop_backdrop_choose_next(backdrop_file);
-    }
-    DBG("new file: %s for Workspace %d", new_backdrop,
-        xfce_workspace_get_workspace_num(workspace));
-
-    if(g_strcmp0(backdrop_file, new_backdrop) != 0) {
-        xfce_backdrop_set_image_filename(backdrop, new_backdrop);
-        g_free(new_backdrop);
-    }
+    /* update the xfconf property */
+    if(new_backdrop != NULL)
+        xfce_workspace_change_backdrop(workspace, backdrop, new_backdrop);
 }
 
 static void
@@ -189,6 +224,14 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
     g_signal_emit(G_OBJECT(user_data), signals[WORKSPACE_BACKDROP_CHANGED], 0, backdrop);
 }
 
+/**
+ * xfce_workspace_set_workspace_num:
+ * @workspace: An #XfceWorkspace.
+ * @GdkScreen: screen the workspace is on.
+ *
+ * Updates the backdrops to correctly display the right settings since GTK/GDK
+ * uses monitor numbers rather than names.
+ **/
 void
 xfce_workspace_monitors_changed(XfceWorkspace *workspace,
                                 GdkScreen *gscreen)
@@ -314,6 +357,149 @@ xfce_workspace_get_property(GObject *object,
     }
 }
 
+/* Attempts to get the backdrop colors from the xfdesktop pre-4.11 format */
+static void
+xfce_workspace_migrate_backdrop_color_style(XfceWorkspace *workspace,
+                                            XfceBackdrop *backdrop,
+                                            guint monitor)
+{
+    XfconfChannel *channel = workspace->priv->channel;
+    char buf[1024];
+    gint pp_len;
+    GValue value = { 0, };
+
+    TRACE("entering");
+
+    /* Use the old property format */
+    g_snprintf(buf, sizeof(buf), "%smonitor%d/",
+               workspace->priv->property_prefix, monitor);
+    pp_len = strlen(buf);
+
+    /* Color style */
+    buf[pp_len] = 0;
+    g_strlcat(buf, "color-style", sizeof(buf));
+    xfconf_channel_get_property(channel, buf, &value);
+
+    if(G_VALUE_HOLDS_INT(&value)) {
+        xfce_backdrop_set_color_style(backdrop, g_value_get_int(&value));
+        g_value_unset(&value);
+    }
+
+    /* first color */
+    buf[pp_len] = 0;
+    g_strlcat(buf, "color1", sizeof(buf));
+    xfconf_channel_get_property(channel, buf, &value);
+
+    if(G_VALUE_HOLDS_BOXED(&value)) {
+        xfce_backdrop_set_first_color(backdrop, g_value_get_boxed(&value));
+        g_value_unset(&value);
+    }
+
+    /* second color */
+    buf[pp_len] = 0;
+    g_strlcat(buf, "color2", sizeof(buf));
+    xfconf_channel_get_property(channel, buf, &value);
+
+    if(G_VALUE_HOLDS_BOXED(&value)) {
+        xfce_backdrop_set_second_color(backdrop, g_value_get_boxed(&value));
+        g_value_unset(&value);
+    }
+
+    /* Fallback to solid if nothing was set anywhere */
+    if(xfce_backdrop_get_color_style(backdrop) == XFCE_BACKDROP_COLOR_INVALID)
+        xfce_backdrop_set_color_style(backdrop, XFCE_BACKDROP_COLOR_SOLID);
+}
+
+/* Attempts to get the backdrop image from the xfdesktop pre-4.11 format */
+static void
+xfce_workspace_migrate_backdrop_image(XfceWorkspace *workspace,
+                                      XfceBackdrop *backdrop,
+                                      guint monitor)
+{
+    XfconfChannel *channel = workspace->priv->channel;
+    char buf[1024];
+    gint pp_len;
+    GValue value = { 0, };
+    const gchar *filename;
+
+    TRACE("entering");
+
+    /* Use the old property format */
+    g_snprintf(buf, sizeof(buf), "%smonitor%d/",
+               workspace->priv->property_prefix, monitor);
+    pp_len = strlen(buf);
+
+    /* Try the old backdrop */
+    buf[pp_len] = 0;
+    g_strlcat(buf, "image-path", sizeof(buf));
+    xfconf_channel_get_property(channel, buf, &value);
+
+    /* Either there was a backdrop to migrate from or we use the backdrop
+     * we provide as a default */
+    if(G_VALUE_HOLDS_STRING(&value))
+        filename = g_value_get_string(&value);
+    else
+        filename = DEFAULT_BACKDROP;
+
+    xfce_backdrop_set_image_filename(backdrop, filename);
+
+    if(G_VALUE_HOLDS_STRING(&value))
+        g_value_unset(&value);
+}
+
+/* Attempts to get the image style from the xfdesktop pre-4.11 format */
+static void
+xfce_workspace_migrate_backdrop_image_style(XfceWorkspace *workspace,
+                                            XfceBackdrop *backdrop,
+                                            guint monitor)
+{
+    XfconfChannel *channel = workspace->priv->channel;
+    char buf[1024];
+    gint pp_len;
+    GValue value = { 0, };
+
+    TRACE("entering");
+
+    /* Use the old property format */
+    g_snprintf(buf, sizeof(buf), "%smonitor%d/",
+               workspace->priv->property_prefix, monitor);
+    pp_len = strlen(buf);
+
+    /* show image */
+    buf[pp_len] = 0;
+    g_strlcat(buf, "image-show", sizeof(buf));
+    xfconf_channel_get_property(channel, buf, &value);
+
+    if(G_VALUE_HOLDS_BOOLEAN(&value)) {
+        gboolean show_image = g_value_get_boolean(&value);
+
+        /* if we aren't showing the image, set the style and exit the function
+         * so we don't set the style to something else */
+        if(!show_image) {
+             xfce_backdrop_set_image_style(backdrop, XFCE_BACKDROP_IMAGE_NONE);
+             g_value_unset(&value);
+             return;
+        }
+
+        g_value_unset(&value);
+    }
+
+    /* image style */
+    buf[pp_len] = 0;
+    g_strlcat(buf, "image-style", sizeof(buf));
+    xfconf_channel_get_property(channel, buf, &value);
+
+    if(G_VALUE_HOLDS_INT(&value)) {
+        XfceBackdropImageStyle style;
+        style = xfce_translate_image_styles(g_value_get_int(&value));
+        xfce_backdrop_set_image_style(backdrop, style);
+        g_value_unset(&value);
+    } else {
+        /* If no value was ever set default to stretched */
+        xfce_backdrop_set_image_style(backdrop, XFCE_BACKDROP_IMAGE_STRETCHED);
+    }
+}
+
 static void
 xfce_workspace_connect_backdrop_settings(XfceWorkspace *workspace,
                                          XfceBackdrop *backdrop,
@@ -363,6 +549,11 @@ xfce_workspace_connect_backdrop_settings(XfceWorkspace *workspace,
     xfconf_g_property_bind(channel, buf, G_TYPE_BOOLEAN,
                            G_OBJECT(backdrop), "backdrop-cycle-enable");
 
+    buf[pp_len] = 0;
+    g_strlcat(buf, "backdrop-cycle-period", sizeof(buf));
+    xfconf_g_property_bind(channel, buf, XFCE_TYPE_BACKDROP_CYCLE_PERIOD,
+                           G_OBJECT(backdrop), "backdrop-cycle-period");
+
     buf[pp_len] = 0;
     g_strlcat(buf, "backdrop-cycle-timer", sizeof(buf));
     xfconf_g_property_bind(channel, buf, G_TYPE_UINT,
@@ -378,6 +569,22 @@ xfce_workspace_connect_backdrop_settings(XfceWorkspace *workspace,
     xfconf_g_property_bind(channel, buf, G_TYPE_STRING,
                            G_OBJECT(backdrop), "image-filename");
 
+    /* If we didn't get a filename try to load one from a previous version */
+    if(xfce_backdrop_get_image_filename(backdrop) == NULL)
+        xfce_workspace_migrate_backdrop_image(workspace, backdrop, monitor);
+
+    /* If we didn't get a proper color style, attempt to get the old one */
+    if(xfce_backdrop_get_color_style(backdrop) == XFCE_BACKDROP_COLOR_INVALID)
+        xfce_workspace_migrate_backdrop_color_style(workspace, backdrop, monitor);
+
+    /* Same for image style, this also deals with 'Auto' */
+    if(xfce_backdrop_get_image_style(backdrop) == XFCE_BACKDROP_IMAGE_INVALID)
+        xfce_workspace_migrate_backdrop_image_style(workspace, backdrop, monitor);
+
+    /* determine if the backdrop will be required to keep a ref of it's pixbuf
+     * when it generates one */
+    xfce_backdrop_set_cache_pixbuf(backdrop, workspace->priv->cache_pixbufs);
+
     g_free(monitor_name);
 }
 
@@ -458,6 +665,14 @@ xfce_workspace_get_workspace_num(XfceWorkspace *workspace)
     return workspace->priv->workspace_num;
 }
 
+/**
+ * xfce_workspace_set_workspace_num:
+ * @workspace: An #XfceWorkspace.
+ * @number: workspace number
+ *
+ * Identifies which workspace this is. Required for XfceWorkspace to get
+ * the correct xfconf settings for its backdrops.
+ **/
 void
 xfce_workspace_set_workspace_num(XfceWorkspace *workspace, gint number)
 {
@@ -466,10 +681,64 @@ xfce_workspace_set_workspace_num(XfceWorkspace *workspace, gint number)
     workspace->priv->workspace_num = number;
 }
 
+/**
+ * xfce_workspace_set_cache_pixbufs:
+ * @workspace: An #XfceWorkspace.
+ * @cache_pixbuf: When TRUE XfceWorkspace will have all it's backdrops keep
+ *                a reference to the current pixbuf.
+ *
+ * This function will control whether XfceWorkspace's backdrops keep a reference
+ * to their respective pixbufs or not. Setting it to TRUE is useful for the
+ * per-workspace-wallpapers, FALSE will save memory for single workspace mode.
+ **/
+void
+xfce_workspace_set_cache_pixbufs(XfceWorkspace *workspace,
+                                 gboolean cache_pixbuf)
+{
+    guint i;
+    guint n_monitors;
+
+    g_return_if_fail(XFCE_IS_WORKSPACE(workspace));
+
+    /* If nothing changed then avoid doing any work */
+    if(workspace->priv->cache_pixbufs == cache_pixbuf)
+        return;
+
+    workspace->priv->cache_pixbufs = cache_pixbuf;
+
+    DBG("cache_pixbuf now %s", cache_pixbuf ? "TRUE" : "FALSE");
+
+    n_monitors = gdk_screen_get_n_monitors(workspace->priv->gscreen);
+
+    /* update all the backdrops */
+    for(i = 0; i < n_monitors && i < workspace->priv->nbackdrops; ++i) {
+        xfce_backdrop_set_cache_pixbuf(workspace->priv->backdrops[i], cache_pixbuf);
+    }
+}
+
+gboolean
+xfce_workspace_get_cache_pixbufs(XfceWorkspace *workspace)
+{
+    g_return_val_if_fail(XFCE_IS_WORKSPACE(workspace), FALSE);
+
+    return workspace->priv->cache_pixbufs;
+}
+
+/**
+ * xfce_workspace_get_backdrop:
+ * @workspace: An #XfceWorkspace.
+ * @monitor: monitor number
+ *
+ * Returns the XfceBackdrop on the specified monitor. Returns NULL on an
+ * invalid monitor number.
+ **/
 XfceBackdrop *xfce_workspace_get_backdrop(XfceWorkspace *workspace,
                                           guint monitor)
 {
-    g_return_val_if_fail(XFCE_IS_WORKSPACE(workspace)
-                         && monitor < workspace->priv->nbackdrops, NULL);
+    g_return_val_if_fail(XFCE_IS_WORKSPACE(workspace), NULL);
+
+    if(monitor >= workspace->priv->nbackdrops)
+        return NULL;
+
     return workspace->priv->backdrops[monitor];
 }
