@@ -62,6 +62,7 @@
 #include "xfdesktop-file-manager-proxy.h"
 #include "xfdesktop-file-utils.h"
 #include "xfdesktop-trash-proxy.h"
+#include "xfdesktop-thunar-proxy.h"
 
 static void xfdesktop_file_utils_add_emblems(GdkPixbuf *pix, GList *emblems);
 
@@ -236,137 +237,6 @@ xfdesktop_file_utils_get_display_name(GFile *file,
     return display_name;
 }
 
-gboolean
-xfdesktop_file_utils_volume_is_present(GVolume *volume)
-{
-    gboolean has_media = FALSE;
-    gboolean is_shadowed = FALSE;
-    GDrive *drive;
-#if GLIB_CHECK_VERSION (2, 20, 0)
-    GMount *mount;
-#endif
-
-    g_return_val_if_fail(G_IS_VOLUME(volume), FALSE);
-
-    drive = g_volume_get_drive (volume);
-    if(drive) {
-        has_media = g_drive_has_media(drive);
-        g_object_unref(drive);
-    }
-
-#if GLIB_CHECK_VERSION (2, 20, 0)
-    mount = g_volume_get_mount(volume);
-    if(mount) {
-        is_shadowed = g_mount_is_shadowed(mount);
-        g_object_unref(mount);
-    }
-#endif
-
-    return has_media && !is_shadowed;
-}
-
-#ifdef HAVE_GIO_UNIX
-static gboolean
-xfdesktop_file_utils_mount_is_internal (GMount *mount)
-{
-    const gchar *point_mount_path;
-    gboolean is_internal = FALSE;
-    GFile *root;
-    GList *lp;
-    GList *mount_points;
-    gchar *mount_path;
-
-    g_return_val_if_fail(G_IS_MOUNT(mount), FALSE);
-
-    /* determine the mount path */
-    root = g_mount_get_root(mount);
-    mount_path = g_file_get_path(root);
-    g_object_unref(root);
-
-    /* assume non-internal if we cannot determine the path */
-    if (!mount_path)
-        return FALSE;
-
-    if (g_unix_is_mount_path_system_internal(mount_path)) {
-        /* mark as internal */
-        is_internal = TRUE;
-    } else {
-        /* get a list of all mount points */
-        mount_points = g_unix_mount_points_get(NULL);
-
-        /* search for the mount point associated with the mount entry */
-        for (lp = mount_points; !is_internal && lp != NULL; lp = lp->next) {
-            point_mount_path = g_unix_mount_point_get_mount_path(lp->data);
-
-            /* check if this is the mount point we are looking for */
-            if (g_strcmp0(mount_path, point_mount_path) == 0) {
-                /* mark as internal if the user cannot mount this device */
-                if (!g_unix_mount_point_is_user_mountable(lp->data))
-                    is_internal = TRUE;
-            }
-
-            /* free the mount point, we no longer need it */
-            g_unix_mount_point_free(lp->data);
-        }
-
-        /* free the mount point list */
-        g_list_free(mount_points);
-    }
-
-    g_free(mount_path);
-
-    return is_internal;
-}
-#endif
-
-
-
-gboolean
-xfdesktop_file_utils_volume_is_removable(GVolume *volume)
-{
-  gboolean can_eject;
-  gboolean can_mount = FALSE;
-  gboolean can_unmount = FALSE;
-  gboolean is_removable = FALSE;
-  gboolean is_internal = FALSE;
-  GDrive *drive;
-  GMount *mount;
-
-  g_return_val_if_fail(G_IS_VOLUME(volume), FALSE);
-
-  /* check if the volume can be ejected */
-  can_eject = g_volume_can_eject(volume);
-
-  /* determine the drive for the volume */
-  drive = g_volume_get_drive(volume);
-  if(drive) {
-      /*check if the drive media can be removed */
-      is_removable = g_drive_is_media_removable(drive);
-
-      /* release the drive */
-      g_object_unref(drive);
-  }
-
-  /* determine the mount for the volume (if it is mounted at all) */
-  mount = g_volume_get_mount(volume);
-  if(mount) {
-#ifdef HAVE_GIO_UNIX
-      is_internal = xfdesktop_file_utils_mount_is_internal (mount);
-#endif
-
-      /* check if the volume can be unmounted */
-      can_unmount = g_mount_can_unmount(mount);
-
-      /* release the mount */
-      g_object_unref(mount);
-  }
-
-  /* determine whether the device can be mounted */
-  can_mount = g_volume_can_mount(volume);
-
-  return (!is_internal) && (can_eject || can_unmount || is_removable || can_mount);
-}
-
 GList *
 xfdesktop_file_utils_file_icon_list_to_file_list(GList *icon_list)
 {
@@ -851,6 +721,59 @@ xfdesktop_file_utils_rename_file(GFile *file,
     }
 }
 
+void
+xfdesktop_file_utils_bulk_rename(GFile *working_directory,
+                                 GList *files,
+                                 GdkScreen *screen,
+                                 GtkWindow *parent)
+{
+    DBusGProxy *thunar_proxy;
+
+    g_return_if_fail(G_IS_FILE(working_directory));
+    g_return_if_fail(GDK_IS_SCREEN(screen) || GTK_IS_WINDOW(parent));
+
+    if(!screen)
+        screen = gtk_widget_get_screen(GTK_WIDGET(parent));
+
+    thunar_proxy = xfdesktop_file_utils_peek_thunar_proxy();
+    if(thunar_proxy) {
+        gchar *directory = g_file_get_path(working_directory);
+        guint nfiles = g_list_length(files);
+        gchar **filenames = g_new0(gchar *, nfiles+1);
+        gchar *display_name = gdk_screen_make_display_name(screen);
+        gchar *startup_id = g_strdup_printf("_TIME%d", gtk_get_current_event_time());
+        GList *lp;
+        gint n;
+
+        /* convert GFile list into an array of filenames */
+        for(n = 0, lp = files; lp != NULL; ++n, lp = lp->next)
+            filenames[n] = g_file_get_basename(lp->data);
+        filenames[n] = NULL;
+
+        xfdesktop_file_utils_set_window_cursor(parent, GDK_WATCH);
+
+        xfdesktop_thunar_proxy_bulk_rename_async(thunar_proxy,
+                                                 directory, (const gchar **)filenames,
+                                                 FALSE, display_name, startup_id,
+                                                 (xfdesktop_thunar_proxy_bulk_rename_reply)xfdesktop_file_utils_async_cb,
+                                                 parent);
+
+        xfdesktop_file_utils_set_window_cursor(parent, GDK_LEFT_PTR);
+
+        g_free(directory);
+        g_free(startup_id);
+        g_strfreev(filenames);
+        g_free(display_name);
+    } else {
+        xfce_message_dialog(parent,
+                            _("Rename Error"), GTK_STOCK_DIALOG_ERROR,
+                            _("The files could not be renamed"),
+                            _("This feature requires a file manager service to "
+                              "be present (such as the one supplied by Thunar)."),
+                            GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT, NULL);
+    }
+}
+
 void
 xfdesktop_file_utils_unlink_files(GList *files,
                                   GdkScreen *screen,
@@ -1194,6 +1117,31 @@ xfdesktop_file_utils_execute(GFile *working_directory,
             uris[n] = g_file_get_uri(lp->data);
         uris[n] = NULL;
 
+        /* If the working_dir wasn't set check if this is a .desktop file
+         * we can parse a working dir from */
+        if(working_dir == NULL) {
+            GFileInfo *info = g_file_query_info(file,
+                                                XFDESKTOP_FILE_INFO_NAMESPACE,
+                                                G_FILE_QUERY_INFO_NONE,
+                                                NULL, NULL);
+
+            if(xfdesktop_file_utils_is_desktop_file(info)) {
+                XfceRc *rc;
+                gchar *path = g_file_get_path(file);
+                if(path != NULL) {
+                    rc = xfce_rc_simple_open(path, TRUE);
+                    if(rc != NULL) {
+                        working_dir = g_strdup(xfce_rc_read_entry(rc, "Path", NULL));
+                        xfce_rc_close(rc);
+                    }
+                    g_free(path);
+                }
+            }
+
+            if(info)
+                g_object_unref(info);
+        }
+
         if(!xfdesktop_file_manager_proxy_execute(fileman_proxy,
                                                  working_dir, uri,
                                                  (const gchar **)uris,
@@ -1449,6 +1397,7 @@ static gint dbus_ref_cnt = 0;
 static DBusGConnection *dbus_gconn = NULL;
 static DBusGProxy *dbus_trash_proxy = NULL;
 static DBusGProxy *dbus_filemanager_proxy = NULL;
+static DBusGProxy *dbus_thunar_proxy = NULL;
 
 gboolean
 xfdesktop_file_utils_dbus_init(void)
@@ -1479,6 +1428,11 @@ xfdesktop_file_utils_dbus_init(void)
                                                            "org.xfce.FileManager",
                                                            "/org/xfce/FileManager",
                                                            "org.xfce.FileManager");
+
+        dbus_thunar_proxy = dbus_g_proxy_new_for_name(dbus_gconn,
+                                                      "org.xfce.FileManager",
+                                                      "/org/xfce/FileManager",
+                                                      "org.xfce.Thunar");
     } else {
         ret = FALSE;
         dbus_ref_cnt = 0;
@@ -1499,6 +1453,12 @@ xfdesktop_file_utils_peek_filemanager_proxy(void)
     return dbus_filemanager_proxy;
 }
 
+DBusGProxy *
+xfdesktop_file_utils_peek_thunar_proxy(void)
+{
+    return dbus_thunar_proxy;
+}
+
 void
 xfdesktop_file_utils_dbus_cleanup(void)
 {
@@ -1509,6 +1469,8 @@ xfdesktop_file_utils_dbus_cleanup(void)
         g_object_unref(G_OBJECT(dbus_trash_proxy));
     if(dbus_filemanager_proxy)
         g_object_unref(G_OBJECT(dbus_filemanager_proxy));
+    if(dbus_thunar_proxy)
+        g_object_unref(G_OBJECT(dbus_thunar_proxy));
 
     /* we aren't going to unref dbus_gconn because dbus appears to have a
      * memleak in dbus_connection_setup_with_g_main().  really; the comments
