@@ -261,11 +261,16 @@ xfdesktop_load_icon_from_desktop_file(XfdesktopRegularFileIcon *regular_icon)
                                               G_KEY_FILE_DESKTOP_KEY_ICON,
                                               NULL);
 
-            /* Create the themed icon for it */
-            if(icon_name) {
+            if(icon_name && g_file_test(icon_name, G_FILE_TEST_IS_REGULAR)) {
+                /* icon_name is an absolute path, create it as a file icon */
+                gicon = g_file_icon_new(g_file_new_for_path(icon_name));
+            } else if(icon_name) {
+                /* otherwise create a themed icon for it */
                 gicon = g_themed_icon_new(icon_name);
-                g_free(icon_name);
             }
+
+            if(icon_name)
+                g_free(icon_name);
         }
 
         /* free key file and in-memory data */
@@ -587,11 +592,8 @@ xfdesktop_regular_file_icon_peek_tooltip(XfdesktopIcon *icon)
 
         size = g_file_info_get_attribute_uint64(info,
                                                 G_FILE_ATTRIBUTE_STANDARD_SIZE);
-#if GLIB_CHECK_VERSION (2, 30, 0)
+
         size_string = g_format_size(size);
-#else
-        size_string = g_format_size_for_display(size);
-#endif
 
         mtime = g_file_info_get_attribute_uint64(info,
                                                  G_FILE_ATTRIBUTE_TIME_MODIFIED);
