@@ -14,8 +14,8 @@
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to the Free Software Foundation,
+ *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -35,8 +35,6 @@
 struct _XfdesktopWindowIconPrivate
 {
     gint workspace;
-    GdkPixbuf *pix;
-    gint cur_pix_size;
     gchar *label;
     WnckWindow *window;
 };
@@ -44,8 +42,9 @@ struct _XfdesktopWindowIconPrivate
 static void xfdesktop_window_icon_finalize(GObject *obj);
 
 static GdkPixbuf *xfdesktop_window_icon_peek_pixbuf(XfdesktopIcon *icon,
-                                                   gint size);
+                                                   gint width, gint height);
 static G_CONST_RETURN gchar *xfdesktop_window_icon_peek_label(XfdesktopIcon *icon);
+static gchar *xfdesktop_window_icon_get_identifier(XfdesktopIcon *icon);
 
 static gboolean xfdesktop_window_icon_activated(XfdesktopIcon *icon);
 static gboolean xfdesktop_window_icon_populate_context_menu(XfdesktopIcon *icon,
@@ -72,6 +71,7 @@ xfdesktop_window_icon_class_init(XfdesktopWindowIconClass *klass)
     
     icon_class->peek_pixbuf = xfdesktop_window_icon_peek_pixbuf;
     icon_class->peek_label = xfdesktop_window_icon_peek_label;
+    icon_class->get_identifier = xfdesktop_window_icon_get_identifier;
     icon_class->activated = xfdesktop_window_icon_activated;
     icon_class->populate_context_menu = xfdesktop_window_icon_populate_context_menu;
 }
@@ -90,9 +90,6 @@ xfdesktop_window_icon_finalize(GObject *obj)
     gchar data_name[256];
     guint16 row, col;
     
-    if(icon->priv->pix)
-        g_object_unref(G_OBJECT(icon->priv->pix));
-    
     g_free(icon->priv->label);
     
     /* save previous position */
@@ -136,43 +133,27 @@ xfdesktop_window_icon_changed_cb(WnckWindow *window,
                                  gpointer user_data)
 {
     XfdesktopWindowIcon *icon = user_data;
-    
-    if(icon->priv->pix) {
-        g_object_unref(G_OBJECT(icon->priv->pix));
-        icon->priv->pix = NULL;
-    }
-    
+
+    xfdesktop_icon_invalidate_pixbuf(XFDESKTOP_ICON(icon));
     xfdesktop_icon_label_changed(XFDESKTOP_ICON(icon));
 }
 
-
-
-
-
 static GdkPixbuf *
 xfdesktop_window_icon_peek_pixbuf(XfdesktopIcon *icon,
-                                 gint size)
+                                 gint width, gint height)
 {
     XfdesktopWindowIcon *window_icon = XFDESKTOP_WINDOW_ICON(icon);
-    
-    if(!window_icon->priv->pix || window_icon->priv->cur_pix_size != size) {
-        if(window_icon->priv->pix)
-            g_object_unref(G_OBJECT(window_icon->priv->pix));
-        
-        window_icon->priv->pix = wnck_window_get_icon(window_icon->priv->window);
-        if(window_icon->priv->pix) {
-            if(gdk_pixbuf_get_width(window_icon->priv->pix) != size) {
-                window_icon->priv->pix = gdk_pixbuf_scale_simple(window_icon->priv->pix,
-                                                                 size,
-                                                                 size,
-                                                                 GDK_INTERP_BILINEAR);
-            } else
-                g_object_ref(G_OBJECT(window_icon->priv->pix));
-            window_icon->priv->cur_pix_size = size;
-        }
+    GdkPixbuf *pix = NULL;
+
+    pix = wnck_window_get_icon(window_icon->priv->window);
+    if(pix) {
+        if(gdk_pixbuf_get_height(pix) != height) {
+            pix = gdk_pixbuf_scale_simple(pix, height, height, GDK_INTERP_BILINEAR);
+        } else
+            g_object_ref(G_OBJECT(pix));
     }
-    
-    return window_icon->priv->pix;
+
+    return pix;
 }
 
 static G_CONST_RETURN gchar *
@@ -186,6 +167,12 @@ xfdesktop_window_icon_peek_label(XfdesktopIcon *icon)
     return window_icon->priv->label;
 }
 
+static gchar *
+xfdesktop_window_icon_get_identifier(XfdesktopIcon *icon)
+{
+    return g_strdup(xfdesktop_window_icon_peek_label(icon));
+}
+
 static gboolean
 xfdesktop_window_icon_activated(XfdesktopIcon *icon)
 {
@@ -205,7 +192,7 @@ xfdesktop_window_icon_populate_context_menu(XfdesktopIcon *icon,
     GtkWidget *amenu = wnck_create_window_action_menu(window_icon->priv->window);
     GtkWidget *mi;
 
-    mi = gtk_menu_item_new_with_mnemonic("_Window Actions");
+    mi = gtk_menu_item_new_with_mnemonic(_("_Window Actions"));
     gtk_menu_item_set_submenu (GTK_MENU_ITEM(mi), amenu);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
     gtk_widget_show_all(mi);
